"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function bitIndexBigInt(v) {
    return v.toString(2).length - 1;
}
exports.bitIndexBigInt = bitIndexBigInt;
function toGindex(index, depth) {
    const anchor = 1n << BigInt(depth);
    if (index >= anchor) {
        throw new Error("index too large for depth");
    }
    return anchor | index;
}
exports.toGindex = toGindex;
function toGindexBitstring(index, depth) {
    const str = index ? index.toString(2) : '';
    if (str.length > depth) {
        throw new Error("index too large for depth");
    }
    else {
        return "1" + str.padStart(depth, "0");
    }
}
exports.toGindexBitstring = toGindexBitstring;
// Get the depth (root starting at 0) necessary to cover a subtree of `count` elements.
// (in out): (0 0), (1 0), (2 1), (3 2), (4 2), (5 3), (6 3), (7 3), (8 3), (9 4)
function countToDepth(count) {
    if (count <= 1) {
        return 0;
    }
    return (count - 1n).toString(2).length;
}
exports.countToDepth = countToDepth;
/**
 * Iterate through Gindexes at a certain depth
 */
function iterateAtDepth(startIndex, count, depth) {
    const anchor = 1n << BigInt(depth);
    if (startIndex + count >= anchor) {
        throw new Error("Too large for depth");
    }
    let i = toGindex(startIndex, depth);
    const last = i + count;
    return {
        [Symbol.iterator]() {
            return {
                next() {
                    if (i < last) {
                        const value = i;
                        i++;
                        return { done: false, value };
                    }
                    else {
                        return { done: true, value: undefined };
                    }
                }
            };
        }
    };
}
exports.iterateAtDepth = iterateAtDepth;
const ERR_INVALID_GINDEX = "Invalid gindex";
function gindexIterator(gindex) {
    if (gindex < 1) {
        throw new Error(ERR_INVALID_GINDEX);
    }
    const bitstring = gindex.toString(2);
    let i = 1;
    const next = () => {
        if (i === bitstring.length) {
            return { done: true, value: undefined };
        }
        const bit = Number(bitstring[i]);
        i++;
        return { done: false, value: bit };
    };
    return {
        [Symbol.iterator]() {
            return { next };
        },
        totalBitLength() {
            return bitstring.length;
        },
        remainingBitLength() {
            return bitstring.length - i;
        },
    };
}
exports.gindexIterator = gindexIterator;
